; PROGRAM NAME	:	Windows Power Meter
; FILE NAME	:	MAX125.asm

; SOFTWARE BY	:	SIMON BRAMBLE
;			simon_bramble@maximhq.com
; DATE		: 	22nd Jan 2000
; DEVELOPMENT TIME	15 Grolsch
; PROCESSOR	:	AT90S4414
; CLOCK SPEED	:	10MHz external clock

; DESCRIPTION
;
; This program simultaneously reads input voltage, input current, output voltage
; and output current around a devices under test (DUT) using the MAX125
; simultaneously sampling ADC. It sends the data, via the MAX3100 and MAX3221, to 
; a PC running a Visual Basic routine. The Windows program displays the data and 
; calculates the DUT efficiency.

.device AT90S4414

;***** I/O Register Definitions

.equ	SPH	=$3e
.equ	SPL	=$3d

.equ	PORTA	=$1b
.equ	DDRA	=$1a
.equ	PINA	=$19
.equ	PORTB	=$18
.equ	DDRB	=$17
.equ	PINB	=$16
.equ	PORTC	=$15
.equ	DDRC	=$14
.equ	PINC	=$13
.equ	PORTD	=$12
.equ	DDRD	=$11
.equ	PIND	=$10

;***** Bit Definitions

.equ	CS	=0
.equ	DIN	=1
.equ	DOUT	=2
.equ	SCLK	=3

.equ	RD	=1
.equ	INT	=2
.equ	WR	=3
.equ	CONVST	=4

.equ 	RAMEND  =$15F				;Last On-Chip SRAM Location
.equ	XRAMEND =$FFFF
.equ	E2END	=$FF
.equ	FLASHEND=$7FF

;***************************************************************************
; PORT DEFINITIONS 
;***************************************************************************

; PORTA : D0-D7 data port + channel select address
;	b7	b6	b5	b4	b3	b2	b1	b0
;	D7	D6	D5	D4	D3/A3	D2/A2	D1/A1	D0/A0	

; PORTC : D8-D13 data port 
;	b7	b6	b5	b4	b3	b2	b1	b0
;			D13	D12	D11	D10	D9	D8

; PORTB : MAX3100 UART Interface
;	b7	b6	b5	b4	b3	b2	b1	b0
;					SCLK	DOUT	DIN	/CS

; PORTD : Control Interface
;	b7	b6	b5	b4	b3	b2	b1	b0
;				/CONVST	/WR	/INT	/RD	/CS

;***************************************************************************
; REGISTER DEFINITIONS
;***************************************************************************

.def	junk		=r16			; junk reg
.def	count		=r17
.def	ch_count	=r18			; channel counter
.def	high_tx		=r19			; upper TX byte for UART
.def	low_tx		=r20			; lower TX byte for UART 
.def	low_rx		=r21			; received data from UART (PC)
.def	high_byte	=r22			; upper data byte from ADC
.def	low_byte	=r23			; lower data byte from ADC
.def	reg1		=r24			; delay routine		
.def	reg2		=r25
.def	reg3		=r26

;***************************************************************************
; MAIN CODE
;***************************************************************************
.cseg
.org	0
	rjmp	reset				; executed on RESET
start:
	rcall	adc_config			; set addressing mode of ADC
	rcall	uart_init
	rcall	PC_cont				; wait for PC control signal
main:
	cbi	PORTD, CONVST
	sbi	PORTD, CONVST			; sample all 4 input channels
						; you may want to put a delay in here
						; to make the interrupt signal easier
						; to see.....
int_loop:
	sbic	PIND, INT			; has conversion finished?
	rjmp	int_loop			; look for interrupt

	ldi	ch_count, 0x04
chan_scan:					; scans ch1-ch4
	rcall	get_data			; load data of each channel

	mov	low_tx, low_byte		; copies low_byte into low_tx
	rcall	wr_data				; shoots low_byte up 232
	rcall	delay10				; 10ms for 232 to occur

	mov	low_tx, high_byte		; copies high_byte into low_tx
	rcall	wr_data				; shoots high_byte up 232
	rcall	delay10				; 10ms for 232 to occur

	dec	ch_count
	brne	chan_scan

	rcall	delay1				; delay of 1 second
	rjmp	main				; and start all over again...doo bee doo bee doo...

;***************************************************************************
; SUBROUTINES
;***************************************************************************

;********************** MICRO INITIALISATION ROUTINE ***********************
reset:						; port definitions and reset
; initialise Stack:

	ldi	junk,high(RAMEND)		; You need this for the 4414 and 8515
	out	SPH,junk			; Don't believe anyone who says otherwise!
	ldi	junk, low(RAMEND)		; 
	out	SPL, junk			; define top of Stack

; initialise ports:

	ldi	junk, 0b00001111		; set A0-A3 as outputs
	out	DDRA, junk

	clr	junk
	out	DDRC, junk			; set C to be all inputs (D0-D5)

	ldi	junk, 0b11111101
	out	DDRB, junk			; PB1 is only input

	ldi	junk, 0b11111011
	out	DDRD, junk			; PD2 is only input

	clr	junk
	out	PORTA, junk			; turn off pull ups
	out	PORTC, junk

	ldi	junk, 0b00000001
	out	PORTB, junk			; all low except CS

	ser	junk
	out	PORTD, junk			; all PORTD is high

	rjmp	start				; return to main program 

;****************************** ADC ROUTINES *******************************
; set up which ADC channels to look at
adc_config:

	cbi	PORTD, CS			; pull CS low
	cbi	PORTD, WR
	ldi	junk, 0b00000011		; Input MUX A / 4-channel
	out	PORTA, junk
	sbi	PORTD, WR	
	sbi	PORTD, CS			; pull CS high

	clr	junk				; set junk all low
	out	DDRA, junk			; configure PORTA as inputs
	out	PORTA, junk			; turn off pull ups

	ret

; acquire data
get_data:
	cbi	PORTD, CS
	cbi	PORTD, RD			; data now available...
	NOP					; allow PORTD to move to PIND
	NOP
	in	high_byte, PINC
	andi	high_byte, 0b00111111		; PORTC bits 6&7 are high Z inputs and can
						; can pick up noise (I've seen it!). This removes it
						; who says digital filtering is tough!
	in	low_byte, PINA
	sbi	PORTD, RD			
	sbi	PORTD, CS
	ret		

;***************************** UART TX ROUTINE  ****************************
uart_init:
	ldi	low_tx, 0b00001011		; 9600, 8 bit, no parity 1 stop bit
wr_config:
	ldi	high_tx, 0b11100000		; No Interrupts from UART
	rjmp	uart_io
rd_config:
	ldi	high_tx, 0b01000000
	rjmp	uart_io
wr_data:
	ldi	high_tx, 0b10000000
	rjmp	uart_io
rd_data:
	ldi	high_tx, 0b00000000
uart_io:
	cbi	PORTB, CS			; start SPI frame and enables chip
	ldi	count, 0x10
uartloop:
	cbi	PORTB, DOUT			; reset Port B, DOUT
	sbrc	high_tx,7			; skip next instruction if MSB is set
	sbi	PORTB, DOUT
	sbi	PORTB, SCLK			; clock data into an SPI device
	clc
	sbic	PINB, DIN
	sec					; set carry bit
	rol	low_rx				; rotates DIN into low_rx LSB
	rol	low_tx				; rotate random data into LSB
	rol	high_tx				; rotate MSB low_ into LSB high_
	cbi	PORTB, SCLK			; clocks data out of an SPI device
	dec	count
	brne	uartloop
	sbi	PORTB, CS			; frames data and shuts down chip
	ret

;************************** PC CONFIRMATION ROUTINE  *************************

PC_cont:
	rcall	rd_data
	cpi	low_rx, 0b00000010
	brne	PC_cont				; keeps lookin' till data in = 5
	ret

;************************* DELAY ROUTINE - 1 SECOND ************************* 

delay1:						; about 1 second
	ldi	reg1, 100
	rjmp	lp1
delay10:					; 10ms delay
	ldi	reg1, 1

lp1:	ldi	reg2, 0x80
lp2:	ldi	reg3,0xFF
lp3:	dec	reg3
	brne	lp3
	dec	reg2
	brne	lp2
	dec	reg1
	brne	lp1
	ret	

;***************************************************************************
; END OF FILE
;***************************************************************************